#!/usr/bin/perl
use strict;
use warnings;
use Getopt::Long;
use DBI;

my %opt = ();
my %legend = (
	"gene_file1=s"			=>      "list of query patterns",
	"gene_file2=s"			=>      "file that contain the true cis-elements",
	"input_number=s"		=>	"number of input files",
	"sequence_length=s"		=>      "score between the query and the cis-element",
	"background_organism=s"		=>      "lenght of the cis-element",
	"output_file=s"			=>      "are wildcard penaltiezed",
	"sql_user=s"			=>	"sql user id",
	"sql_pass=s"			=>	"sql user password",
	"sql_db=s"			=>	"sql table",
);
GetOptions(\%opt, "help!", keys %legend);

my($gene_file1)			= "";
my($gene_file2)			= "";
my($input_number)		= "";
my($sequence_length)		= "";
my($background_organism)	= "";
my($output_file)		= "";
my($sql_user)			= "";
my($sql_pass)			= "";
my($sql_db)			= "";

if (exists($opt{'gene_file1'}))		{ $gene_file1 = $opt{'gene_file1'}; 				} 
else 					{ die "Missing parameter: gene_file1"; 				}
if (exists($opt{'input_number'}))	{ $input_number = $opt{'input_number'}; 			} 
else 					{ die "Missing parameter: input_number"; 			}

if ($input_number == 2) {
	if (exists($opt{'gene_file2'})) 	 { $gene_file2 = $opt{'gene_file2'}; 			} 
	else { die "Missing parameter: gene_file2"; }
}

if (exists($opt{'sequence_length'}))	{ $sequence_length = $opt{'sequence_length'}; 			} 
else 					{ die "Missing parameter: sequence_length"; 			}
if (exists($opt{'background_organism'})){ $background_organism = $opt{'background_organism'}; 		} 
else 					{ die "Missing parameter: background_organism"; 		}
if (exists($opt{'output_file'})) 	{ $output_file = $opt{'output_file'}; 				} 
else 					{ die "Missing parameter: output_file";	 			}
if (exists($opt{'sql_user'}))		{ $sql_user = $opt{'sql_user'};					} 
else 					{ die "Missing parameter: sql_user"; 				}
if (exists($opt{'sql_pass'}))		{ $sql_pass = $opt{'sql_pass'}; 				} 
else 					{ die "Missing parameter: sql_pass"; 				}
if (exists($opt{'sql_db'}))		{ $sql_db = $opt{'sql_db'}; 					} 
else 					{ die "Missing parameter: sql_db"; 				}

&main($gene_file1, $gene_file2, $input_number, $sequence_length, $background_organism, 
"$output_file\.html", $sql_user, $sql_pass, $sql_db);

eval 'use lib "/data/backup/zope_data/poxo/COMMON"';
eval 'use CreateCheckfile';
eval '&CreateCheckfile("$output_file\.checkme")';

exit;

##########################################################################################################
# main interface
sub main {
	my($gene_file1, $gene_file2, $input_number, $sequence_length, $background_organism,  
	$output_file, $sql_user, $sql_pass, $sql_db)							= @_;
	my(@gene_set1)											= ();
	my(@gene_set1_temp)										= ();
	my(@gene_set2)											= ();
	my(@gene_set2_temp)										= ();
	my(@unlink_file)										= ();
	
	my(@gene_set1_found)										= ();
	my(@gene_set1_miss)										= ();
	my(@gene_set2_found)										= ();
	my(@gene_set2_miss)										= ();
	
	my($line)											= "";
	
	open(GENE_SET1, "$gene_file1");
	@gene_set1_temp = <GENE_SET1>;
	close GENE_SET1;
	
	for(my($i) = 0; $i < @gene_set1_temp; ++$i) {
		$line = $gene_set1_temp[$i];
		$line =~ s/^\s+|\s+$//gi;
		push(@gene_set1, split(/\s/, $line));	
	}
	
	&RetrieveSequence(\@gene_set1, $sequence_length, $background_organism, $sql_user, $sql_pass, $sql_db, 
			  \@gene_set1_found, \@gene_set1_miss );
			  
	if($input_number == 2) {
		open(GENE_SET2, "$gene_file2");
		@gene_set2_temp = <GENE_SET2>;
		close GENE_SET2;

		for(my($i) = 0; $i < @gene_set2_temp; ++$i) {
			$line = $gene_set2_temp[$i];
			$line =~ s/^\s+|\s+$//gi;
			push(@gene_set2, split(/\s/, $line));
		}

		&RetrieveSequence(\@gene_set2, $sequence_length, $background_organism, $sql_user, $sql_pass, $sql_db,
				  \@gene_set2_found, \@gene_set2_miss);
	}

	&PrintHeader($input_number, $sequence_length, $background_organism, $output_file);
	&PrintSequences(\@gene_set1_found, \@gene_set1_miss, \@gene_set2_found, \@gene_set2_miss, $output_file);
	&PrintFooter($output_file);

	#push(@unlink_file, $gene_file1);
	#push(@unlink_file, $gene_file2);
	#unlink(@unlink_file);
};
##########################################################################################################
# Print header section
sub PrintFooter {
	my($output_file)										= @_;
	open(OUTPUT, ">>$output_file");

	print OUTPUT "</form><br>\n";
	print OUTPUT "<dtml-var link_collection>\n";
	print OUTPUT "<dtml-var poxo_footer>\n";
	
	close OUTPUT;
};
##########################################################################################################
# Print header section
sub PrintHeader {
	my($input_number, $sequence_length, $background_organism, $output_file)				= @_;

	open(OUTPUT, ">$output_file");

	print OUTPUT "<dtml-var poxo_header>\n";
	print OUTPUT "<form name='retriever' target='_blank' method='post' enctype='multipart/form-data'>\n";
	print OUTPUT "<input type='hidden' name='input_number' size='0' value='$input_number'>\n"; 
	print OUTPUT "<input type='hidden' name='sequence_length' size='0' value='$sequence_length'>\n";
	print OUTPUT "<input type='hidden' name='background_organism' size='0' value='$background_organism'>\n";
	
	close OUTPUT;
};
##########################################################################################################
# Sequence retrieval
sub PrintSequences {
	my($gene_set1_found, $gene_set1_miss, $gene_set2_found, $gene_set2_miss, $output_file)		= @_;

	open(OUTPUT, ">>$output_file");
	
	print OUTPUT "<table border='1' cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='B0C4DE' width='95%'>\n";
	print OUTPUT "<tr>\n";
	print OUTPUT "<td align='center' width='50%'>\n";
	print OUTPUT "<a href='help#p4' target ='_blank'>Sequences of the genes of the first cluster</a>\n";
	print OUTPUT "</td>\n";
	print OUTPUT "<td align='center' width='50%'>\n";
	print OUTPUT "<a href='help#p4' target ='_blank'>Sequences of the genes of the second cluster</a>\n";
	print OUTPUT "</td>\n";
	print OUTPUT "</tr>\n";
	print OUTPUT "<tr>\n";
	print OUTPUT "<td align='center' width='50%'>\n";
	print OUTPUT "<TEXTAREA name='sequence_set1' rows='10' cols='30' wrap='virtual' align='left'>\n";
	
	for(my($i) = 0; $i <  scalar(@$gene_set1_found); ++$i) {
			print OUTPUT "@$gene_set1_found[$i]\n";
	}
	
	print OUTPUT "</TEXTAREA>\n";
	print OUTPUT "</td>\n";
	
	print OUTPUT "<td align='center' width='50%'>\n";
	print OUTPUT "<TEXTAREA name='sequence_set2' rows='10' cols='30' wrap='virtual' align='left'>\n";
	
	for(my($i) = 0; $i <  scalar(@$gene_set2_found); ++$i) {
		print OUTPUT "@$gene_set2_found[$i]\n";
	}
					
	print OUTPUT "</TEXTAREA>\n";
	print OUTPUT "</td>\n";
	print OUTPUT "</tr>\n";
	print OUTPUT "<tr>\n";
	print OUTPUT "<td align='center' width='50%'><font color='000000'>Unreturned genes of the first cluster</font></td>\n";
	print OUTPUT "<td align='center' width='50%'><font color='000000'>Unreturned genes of the second cluster</font></td>\n";
	print OUTPUT "</tr>\n";

	print OUTPUT "<tr>\n";
	print OUTPUT "<td align='center' width='50%'><font color='000000'>\n";
	if (scalar(@$gene_set1_miss) == 0) {
		print OUTPUT "None\n";
	} else {
		for(my($i) = 0; $i < (scalar(@$gene_set1_miss) -1); ++$i) {
			print OUTPUT "@$gene_set1_miss[$i], ";
		}
		print OUTPUT "@$gene_set1_miss[(scalar(@$gene_set1_miss) -1)]";
	}
	print OUTPUT "</font></td>\n";
	
	print OUTPUT "<td align='center' width='50%'><font color='000000'>\n";
	if (scalar(@$gene_set2_miss) == 0) {
		print OUTPUT "None\n";
	} else {
        	for(my($i) = 0; $i < (scalar(@$gene_set2_miss) -1); ++$i) {
			print OUTPUT "@$gene_set2_miss[$i], ";
		}
		print OUTPUT "@$gene_set2_miss[(scalar(@$gene_set2_miss) -1)]";
	}
	print OUTPUT "</font></td>\n";
	print OUTPUT "</tr>\n";	
	
	print OUTPUT "</table>\n";
	close OUTPUT;
};
##########################################################################################################
# Sequence retrieval
sub RetrieveSequence {
	my($gene_set, $sequence_lenght, $background_organism, $sql_user, $sql_pass, $sql_db, 
	   $sequence_set_found, $sequence_set_miss)							= @_;
	my($line)											= "";
	my($sequence)											= "";
	my($sequence_ret_length)									= "";
	my($is_found)											= 0;
	
	my($sock)	= "/var/lib/mysql/mysql.sock";                 # corrected 29.5.2008
	my($dsn) 	= "DBI:mysql:$sql_db;mysql_socket=$sock";
	my($dbh) 	= DBI->connect($dsn, $sql_user, $sql_pass);
	my($sth) 	= "";

	for(my($i) = 0; $i < scalar(@$gene_set); ++$i) {
		$line = @$gene_set[$i];
		$line =~ s/^>+//gi;
		$line =~ s/^\s+|\s+$//gi;
		$line =~ s/^>+//gi;
		$line = uc($line);
		if(length($line) > 0) {
			$is_found = 0;
			$sth = $dbh->prepare("Select Sequence from $background_organism where ucase(Info) like '\%\|$line\|\%';");
			$sth->execute();
			
			while (my $ref = $sth->fetchrow_hashref()) {
				$sequence = ($ref->{'Sequence'});
				$sequence_ret_length = length($sequence);
				
				if ($sequence_ret_length >= $sequence_length ) {
					$sequence = substr($sequence, ($sequence_ret_length - $sequence_length), $sequence_length);
				}
				
				$is_found = 1;
			}

			if ($is_found == 1) {
				push(@$sequence_set_found, ">$line");
				push(@$sequence_set_found, $sequence);
			} else {
				push(@$sequence_set_miss, $line);
			}
		}
	}
	
	$dbh->disconnect();
};
##########################################################################################################

